/***************************************************************************
                          calltreedlg.h  -  description
                             -------------------
    begin                : Fri Jul 18 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CALLTREEDLG_H
#define CALLTREEDLG_H

#include <qwidget.h>
#include <qlistview.h>
#include <calltreelayout.h>
#include "cscopefrontend.h"

class QueryResultsMenu;

/**
 * Displays a hierarchical tree of calling/called functions.
 * Each item in the tree is composed of the function name, the path of the
 * source file in which it is defined, the line number and the full text of
 * that line.
 * @author Elad Lahav
 */

class CallTreeDlg : public CallTreeLayout
{
   Q_OBJECT
   
public: 
	CallTreeDlg(QWidget* pParent = 0, const char* szName = 0);
	~CallTreeDlg();
	
	void setRoot(const QString&);
	bool load(const QString&, const QString&);
	void store(const QString&);
	
	/** Returns Call Tree filename */
	QString getFileName() { return m_sFileName; }

signals:
	/**
	 * Emitted when the user selects an item in the tree view.
	 * @param	sPath	The full path of the file associated with the item
	 * @param	nLine	The line number associated with the item
	 */
	void lineRequested(const QString& sPath, uint nLine);
	
	/**
	 * Emitted when the user closes the tree view.
	 */
	void closed(const CallTreeDlg*);

protected:
	virtual void closeEvent(QCloseEvent*);

protected slots:
	virtual void slotSaveClicked();
	virtual void slotZoomInClicked();
	virtual void slotZoomOutClicked();
	virtual void slotRotateClicked();
	
private:
	QString m_sQueriedFunc;
	bool m_bCalled;
	
	/** A Cscope process to use for running queries. */
	CscopeFrontend* m_pCscope;
	
	/** Displays query progress information. */
	CscopeProgress m_progress;
	
	/** File name to save the call tree */
	QString m_sFileName;
	
	/** A popup-menu for manipulating query result items. */
	QueryResultsMenu* m_pQueryMenu;
		
	/** File Name index for the file name generation */
	static int s_nFileNameIndex;
	
	void query(const QString&, bool);

private slots:
	void slotDataReady(FrontendToken*);
	void slotProgress(int, int);
	void slotFinished(uint);
	void slotQueryCalled(const QString&);
	void slotQueryCalling(const QString&);
	void slotFindDef(const QString&);
};

#endif
