/***************************************************************************
                            codegenerator.cpp
                             -------------------
    begin                : Fri Jun 10 2005
    copyright            : (C) 2005 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "codegenerator.h"

// TODO:
// Why are these factors correct?
// Is there a better way to get the required coordinates from the given ones?
#define FACTOR			5 / 4
#define TO_QPOINT(_pt)	QPoint(_pt.x * FACTOR, (s_box.UR.y - _pt.y) * FACTOR)
#define TO_QSIZE(_box)	QSize(_box.UR.x * FACTOR, _box.UR.y * FACTOR)

codegen_t CodeGenerator::s_cg = {
	NULL /* reset */,
	NULL /* begin_job */,
	NULL /* end_job */,
	beginGraph,
	NULL /* end_graph */,
	NULL /* begin_page */,
	NULL /* end_page */ ,
	NULL /* begin_layer */,
	NULL /* end_layer */,
	NULL /* begin_cluster */,
	NULL /* end_cluster */,
	NULL /* begin_nodes */,
	NULL /* end_nodes */,
	NULL /* begin_edges */,
	NULL /* end_edges */,
	beginNode,
	endNode,
	beginEdge,
	endEdge,
	NULL /* begin_context */,
	NULL /* end_context */,
	NULL /* begin_anchor */,
	NULL /* end_anchor */,
	setFont,
	textLine,
	setPenColor,
	setFillColor,
	NULL /* set_style */,
	NULL /* ellipse */,
	polygon,
	bezierCurve,
	NULL /* polyline */,
	false,
	NULL /* comment */,
	NULL /* textsize */,
	NULL /* user_shape */,
	NULL /* userShapeSize */
};

GraphWidget* CodeGenerator::s_pWidget;
box CodeGenerator::s_box;

/**
 * Attaches the given graph to the code generator before a drawing operation
 * is initiated.
 * @param	pWidget	The widget that should be used for drawing
 */
void CodeGenerator::bind(GraphWidget* pWidget)
{
	
	s_pWidget = pWidget;
	s_pWidget->getContext()->codegen = &s_cg;
}

/**
 * Called at the beginning of the drawing process.
 * Resizes the widget to fit the graph's bounding rectangle.
 * @param	br	The graph's bounding rectangle
 */
void CodeGenerator::beginGraph(GVC_t*, graph_t*, box br, point)
{
	s_box = br;
	s_pWidget->resize(TO_QSIZE(s_box));
}

/**
 * Called before a node is drawn.
 * @param	pNode	The drawn node
 */
void CodeGenerator::beginNode(node_t* pNode)
{
	s_pWidget->startNode(pNode);
}

/**
 * Called after a node has been drawn.
 */
void CodeGenerator::endNode()
{
	s_pWidget->endNode();
}

/**
 * Called before an edge is drawn.
 * @param	pEdge	The drawn edge
 */
void CodeGenerator::beginEdge(edge_t* pEdge)
{
	s_pWidget->startEdge(pEdge);
}

/**
 * Called after an edge has been drawn.
 */
void CodeGenerator::endEdge()
{
	s_pWidget->endEdge();
}

/**
 * Determines the font that should be used for drawing text.
 * @param	szFamily	The font's family name
 * @param	dSize		The font's size
 */
void CodeGenerator::setFont(char* szFamily, double dSize)
{
	s_pWidget->setFont(QFont(szFamily, (int)dSize));
}

/**
 * Draws some text at the given position.
 * @param	pt			The point where the text should be drawn
 * @param	pTextLine	Contains the text to draw
 */
void CodeGenerator::textLine(point pt, textline_t* pTextLine)
{
	s_pWidget->text(pTextLine->str, TO_QPOINT(pt));
}

/**
 * Sets the pen used to draw lines.
 * @param	szColor	The name of the colour to use
 */
void CodeGenerator::setPenColor(char* szColor)
{
	s_pWidget->setPen(QColor(szColor));
}

/**
 * Sets the brush used to fill shapes.
 * @param	szColor	The name of the colour to use
 */
void CodeGenerator::setFillColor(char* szColor)
{
	s_pWidget->setBrush(QColor(szColor));
}

/**
 * Draws a polygon using the given points.
 * @param	pArray	An array of points
 * @param	nSize	The size of the array
 */
void CodeGenerator::polygon(point* pArray, int nSize, int)
{
	QPointArray arrPoints(nSize + 1);
	int i;
	
	// Create an array of QPoint objects
	for (i = 0; i < nSize; i++)
		arrPoints.setPoint(i, TO_QPOINT(pArray[i]));
	
	// Need to close the shape to get a polygon
	arrPoints.setPoint(nSize, TO_QPOINT(pArray[0]));
	
	// Draw the polygon
	s_pWidget->polygon(arrPoints);
}

/**
 * Draws a cubic Bezier curve using the given control points.
 * @param	pArray	An array of control points
 * @param	nSize	The size of the array
 */
void CodeGenerator::bezierCurve(point* pArray, int nSize, int, int)
{
	QPointArray arrPoints(nSize);
	int i;
	
	// Create an array of QPoint objects
	for (i = 0; i < nSize; i++)
		arrPoints.setPoint(i, TO_QPOINT(pArray[i]));
	
	// Draw the curve
	s_pWidget->curve(arrPoints);
}
