/***************************************************************************
                            codegenerator.h
                             -------------------
    begin                : Fri Jun 10 2005
    copyright            : (C) 2005 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CODEGENERATOR_H
#define CODEGENERATOR_H

#include "graphviz.h"
#include "graphwidget.h"

/**
 * Implements a code generator for a graphviz rendering environment, as
 * defined by a codegen_t structure.
 * The codegen_t structure is defined as an aggregate of function pointers.
 * Therefore, this class has to implement all functions as static methods.
 * A GraphWidget object uses this class through the draw() function. This
 * function sets the calling widget as the current target for its drawing
 * methods. It then tells the graphviz engine to draw the graph stored in the
 * widget. The engine calls the static drawing methods with instructions on
 * how to draw the graph. These instructions are delegated back to the 
 * GraphWidget object, which turns them into canvas items.
 * @author Elad Lahav
 */
class CodeGenerator
{
public:
	static void bind(GraphWidget*);
	
private:
	/** Holds pointers to the class's static drawing methods. */
	static codegen_t s_cg;
	
	/** The last widget to call draw(). */
	static GraphWidget* s_pWidget;
	
	/** The bounding rectangle of the current graph. */
	static box s_box;
	
	static void beginGraph(GVC_t*, graph_t*, box, point);
	static void beginNode(node_t*);
	static void endNode();
	static void beginEdge(edge_t*);
	static void endEdge();
	static void setFont(char*, double);
	static void textLine(point, textline_t*);
	static void setPenColor(char*);
	static void setFillColor(char*);
	static void polygon(point*, int, int);
	static void bezierCurve(point*, int, int, int);
};

#endif
